@@ -1,7 +1,7 @@
 #
-# remt - reMarkable tablet command-line tools
+# remt - note-taking support tool for reMarkable tablet
 #
-# Copyright (C) 2018-2020 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2018-2021 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ def page_strokes(items: Items) -> PageStrokes:
     items = (v for v in items if is_page(v) or is_highlighter(v))
 
     # split into (page, strokes)
-    yield from split(is_page, items)
+    yield from split(is_page, items)  # type: ignore
 
 def page_selections(strokes: tp.Tuple[Stroke]) -> tp.List[TextSelection]:
     """
