@@ -1,7 +1,7 @@
 #
-# remt - reMarkable tablet command-line tools
+# remt - note-taking support tool for reMarkable tablet
 #
-# Copyright (C) 2018-2020 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2018-2021 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,9 @@ reMarkable tablet lines format parser.
 """
 
 import contextvars
+import cytoolz.itertoolz as itz  # type: ignore
 import enum
+import json
 import logging
 import struct
 import typing as tp
@@ -81,18 +83,23 @@ FMT_STROKE = Parser[TYPE_STROKE]('<IIIfI')
 FMT_STROKE_V5 = Parser[TYPE_STROKE_V5]('<IIIfII')
 FMT_SEGMENT = Parser[TYPE_SEGMENT]('<ffffff')
 
-def parse(data: tp.BinaryIO, page_number: int) -> rdata.Items:
+def parse(
+        data: tp.BinaryIO,
+        page_number: int,
+        highlights: tp.List[rdata.Stroke]
+    ) -> rdata.Items:
     """
     Parse reMarkable file and return iterator of document items.
 
     :param data: reMarkable file data.
     :param page_number: Page number for which data is parsed.
+    :param highlights: Highlighter information parsed from JSON files.
     """
     header, ver, *_ = FMT_HEADER_PAGE.parse(data)
     assert header.startswith(HEADER_START), 'header is {!r}'.format(header)
     CTX_PARSER.set(Version(ver))
     logger.info('file format version {}'.format(chr(ver)))
-    yield from parse_page(data, page_number)
+    yield from parse_page(data, page_number, highlights)
 
 def empty_page(page_number: int) \
         -> tp.Iterable[tp.Union[rdata.Page, rdata.PageEnd]]:
@@ -104,8 +111,11 @@ def empty_page(page_number: int) \
     yield rdata.Page(page_number)
     yield rdata.PageEnd(page_number)
 
-def parse_page(data: tp.BinaryIO, page_number: int) \
-        -> rdata.Items:
+def parse_page(
+        data: tp.BinaryIO,
+        page_number: int,
+        highlights: tp.List[rdata.Stroke]
+    ) -> rdata.Items:
     """
     Parse page from reMarkable table file.
     """
@@ -114,6 +124,7 @@ def parse_page(data: tp.BinaryIO, page_number: int) \
 
     yield rdata.Page(page_number)
     yield from flatten(items)
+    yield from highlights
     yield rdata.PageEnd(page_number)
 
 def parse_layer(n_layer: int, data: tp.BinaryIO) -> rdata.Items:
@@ -129,15 +140,24 @@ def parse_stroke(n_stroke: int, data: tp.BinaryIO) \
 
     ver = CTX_PARSER.get()
     if ver == Version.VERSION_5:
-        pen, color, unk1, width, unk2, n = FMT_STROKE_V5.parse(data)
+        pen, color_idx, unk1, width, unk2, n = FMT_STROKE_V5.parse(data)
     elif ver == Version.VERSION_3:
-        pen, color, a, width, n = FMT_STROKE.parse(data)
+        pen, color_idx, a, width, n = FMT_STROKE.parse(data)
+    else:
+        assert False
+
+    # fix color index for highlighter, default it to yellow if black, gray
+    # or white was stored by remarkable
+    if rdata.Pen.is_highlighter(pen) and color_idx in (0, 1, 2):
+        color = rdata.ColorIndex.YELLOW
+    else:
+        color = rdata.ColorIndex(color_idx)
 
     segments = [parse_segment(i, data) for i in range(n)]
     stroke = rdata.Stroke(
         n_stroke,
         rdata.Pen(pen),
-        rdata.ColorIndex(color),
+        color,
         width,
         segments
     )
@@ -148,4 +168,30 @@ def parse_segment(n_seg: int, data: tp.BinaryIO) -> rdata.Segment:
     x, y, speed, direction, width, pressure = FMT_SEGMENT.parse(data)
     return rdata.Segment(n_seg, x, y, speed, direction, width, pressure)
 
+def parse_highlights(fin: tp.TextIO) -> tp.Iterator[rdata.Stroke]:
+    data = json.load(fin)
+    highlights = itz.concat(v for v in data.get('highlights', []))
+    items = (parse_highlight_item(h) for h in highlights)
+    yield from items
+
+def parse_highlight_item(highlight: dict[str, tp.Any]) -> rdata.Stroke:
+    to_seg = lambda x, y, w: rdata.Segment(0, x, y, 0, 0, w, 1)
+    items = (
+        (v['x'], v['y'], v['width'], v['height'] / 2)
+        for v in highlight['rects']
+    )
+    segments = itz.concat(
+        (to_seg(x, y + h, h), to_seg(x + w, y + h, h))  # type: ignore
+        for x, y, w, h in items
+    )
+    color = highlight.get('color', rdata.ColorIndex.YELLOW)
+    return rdata.Stroke(
+        0,
+        rdata.Pen.HIGHLIGHTER_2,
+        color,
+        rdata.PEN_WIDTH_HIGHLIGHTER,
+        list(segments),
+        text=highlight['text']
+    )
+
 # vim: sw=4:et:ai
